;TrackLoad
;Mark Spankus 2/2021
;
;Simple track loader for custom program that does not need OS65D
;This program is stored on track 0 in normal OSI boot format starting with 22 00 01 <program data>
;(The load address $2200 is hard-wired into most OSI boot ROMS)
;The program you want to load follows on the next tracks
;with load address hi, load address low, # of pages, <# of pages of data>
;more tracks may follow with the same format
;the loader is located in zero page, so tracks may load at any address including $2200
;The last track specifies execution address address hi, address lo, 0  (3 bytes total)

; $43,$57, BCDRK, $58, $76, $01, #SEC_PAGES, $47,$53

;ROM routine loads 1st track, and jumps to starting address ($2200)
DDPIA = $C000
DDACIA= $C010
LODMOR= $0000


*=$2200		
	LDX #$00
INSTAL
	LDA PROGLOC,X
	STA $00,X
	INX
	CPX #<LODEND
	BNE INSTAL
	JMP LODMOR
PROGLOC=*

*=LODMOR  ;this causes a binary relocation error which we ignore because it is desired behavior (packed binary)

;increment track
	LDA DDPIA+2
	AND #$FB	; 1111 1011   DIR=INWARDS TO 39/76
	STA DDPIA+2	; SET 'DIR' PIN
	JSR SDELAY	;short delay
	AND #$F7	; 1111 0111
	STA DDPIA+2	; SET 'STEP' LOW
	JSR SDELAY
	ORA #8		; 0000 1000
	STA DDPIA+2	; SET 'STEP' HIGH
	LDX #$18
	JSR LDELAY   ;longer delay
	LDX #$7F	 ;load head
	STX DDPIA+2
	JSR LDELAY	 ;delay 320 ms
IDXSRT	
	LDA DDPIA
	BMI IDXSRT	 ;wait for index start
IDXEND
	LDA DDPIA
	BPL IDXEND	 ;wait for index end
	LDA #$03
	STA DDACIA	 ;reset ACIA
	LDA #$58
	STA DDACIA	 ;/1 RTS hi, no irq
HDR1	
	JSR RDISK
	CMP #$76
	BNE HDR1
	JSR RDISK   ; get sector start code 01
	CMP #$01
	BNE HDR1
	JSR RDISK   ;read number of pages/65D sectors
	JSR RDISK   ;read number of pages	
	STA $FF 	;store # pages to process (-3)
	BEQ GOEND	
	CMP #$0E	;either 0 or invalid sector length causes exit
	BCS GOEND	

	JSR RDISK	;read start addr hi
	STA $FE	 	
	JSR RDISK	;read start addr lo  
	SEC
	SBC #$03
	STA $FD
	BCS NODEC
	DEC $FE
NODEC
;	JSR RDISK
;	STA $FF	 	;read num pages

	LDY #$03	
RLOOP
	JSR RDISK	 ;read data
	BCC RERROR
	STA ($FD),Y
	INY
	BNE RLOOP
	INC $FE
	DEC $FF
	BNE RLOOP
	LDA #$FF    ;disable drive            
	STA DDPIA+2
	JMP LODMOR
RERROR
	JMP	($FFFC)	;Exit to RESET vector
LDELAY      
	LDY #$F8	;long delay FF85
	DEY
	BNE LDELAY+2
	EOR $FF,X
	DEX
	BNE LDELAY
SDELAY
	CLC
	RTS
RDISK = *		;Read character from disk, returned in A, carry set=OK clear=error
	LDA DDPIA	;check for index during read
	BPL SDELAY
	LDA DDACIA	;wait for ACIA char
	LSR A
	BCC RDISK
	LDA DDACIA+1
	RTS
GOEND
	LDA #$80
	STA DDPIA+2 ;unload head
	LDX #$05
	LDY #$00
C3JMP
	LDA <GOJMP,X 
	STA $00,X	;set C1P Warm Start
	STA $120,Y	;set 65A 'Go' location for restart (reverse addr order from jmp)
	DEY
	DEX
	BPL C3JMP
GOJMP=*
	JMP $0300
	.BYTE $FD,$04 ;additional bytes for 65A 'Go'  //0129, Y=0, X=0, A=0, P=4, k=FD PCh PCl

LODEND
	.BYTE 0
	

	